class CrossValidatorBinException(Exception):
    pass


class TimeSeriesCrossValidatorBinException(Exception):
    pass


class DimensionsMismatchException(Exception):
    pass


class TrainTestSplitException(Exception):
    pass


class InitialPopulationUndefinedException(Exception):
    pass
